Package: simglm
Type: Package
Authors@R: person("Brandon", "LeBeau", email = "lebebr01+simglm@gmail.com",
     role = c("aut", "cre"))
Version: 0.8.9
Date: 2022-02-06
License: MIT + file LICENSE
Title: Simulate Models Based on the Generalized Linear Model
Description: Simulates regression models,
    including both simple regression and generalized linear mixed
    models with up to three level of nesting. Power simulations that are
    flexible allowing the specification of missing data, unbalanced designs,
    and different random error distributions are built into the package.
Depends: R (>= 3.6.0)
Imports: stats, methods, rlang, dplyr, purrr, broom, future.apply
Suggests: knitr, lme4, nlme, testthat, shiny, e1071, ggplot2, tidyr,
        geepack, rmarkdown, future, splines, covr
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.1.2
Author: Brandon LeBeau [aut, cre]
Maintainer: Brandon LeBeau <lebebr01+simglm@gmail.com>
URL: https://github.com/lebebr01/simglm
BugReports: https://github.com/lebebr01/simglm/issues
NeedsCompilation: no
Packaged: 2022-02-07 04:20:41 UTC; bleb
Repository: CRAN
Date/Publication: 2022-02-07 08:20:02 UTC
Built: R 4.6.0; ; 2026-04-30 16:46:43 UTC; unix
