# Generated by roxygen2: do not edit by hand

export(compute_statistics)
export(correlate_variables)
export(desireVar)
export(dropout_missing)
export(extract_coefficients)
export(generate_missing)
export(generate_response)
export(mar_missing)
export(missing_data)
export(model_fit)
export(parse_correlation)
export(parse_crossclass)
export(parse_formula)
export(parse_power)
export(parse_randomeffect)
export(parse_varyarguments)
export(random_missing)
export(rbimod)
export(replicate_simulation)
export(run_shiny)
export(sim_continuous2)
export(sim_factor2)
export(sim_time)
export(simglm)
export(simulate_error)
export(simulate_fixed)
export(simulate_heterogeneity)
export(simulate_knot)
export(simulate_randomeffect)
export(transform_outcome)
importFrom(dplyr,`%>%`)
importFrom(dplyr,count)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,quo)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(future.apply,future_replicate)
importFrom(methods,selectMethod)
importFrom(purrr,modify_if)
importFrom(rlang,syms)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,model.matrix)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
